% Input:
% eA: Matrix (expm(A))
% X0: function handle (Support function of the initial set)
% normvalue: scalar
% t: double (stepsize)
% L: Matrix (row-wise direction vectors)
% adddimension: vector

% Output:
% omega0: function handle (Support function overapproximating the initial
% set)
% alpha: scalar

% Description:
% Calculates the initial set for the reachability analysis with picewise
% constant inputs u (fr algoConstU)

function [omega0,alpha] = calcPinitConstU(eA,X0,normvalue,t,L,adddimension)
neA = norm(eA,normvalue);

if ~isempty(adddimension)
    L = [L; zeros(2,size(L,2)-1), adddimension];
end;

alpha = ( exp(t*neA) -1 -t*neA) * calcR(zeros(size(eA,1),1),normvalue,X0,L,'active-set',[],[]);
omega0 = @(l) max(X0(l),X0(eA'*l))+ alpha*norm(l,2);
